#ifndef PARTICLETRACKER_HH
#define PARTICLETRACKER_HH
#
#include <vector>
#include "Vector.hh"
#include "grid.hh"
#include "particletracker_utilities.hh"

class ParticleTracker
{
public:
  typedef PressureVelocityConverter ConvectionField;
  typedef ConstantDispersion DispersionField;

  class Particle
  {
  public:
    Vector position;
    double time;
	
    Particle() 
      : 
	  position( Grid::dimensions, 0 )
	  , time(0)
    {}

  };

private:
  const Grid & grid;
  const ConvectionField & convection_field;
  const DispersionField & dispersion_field;
  std::vector<Particle> particles;
  const Vector extent;
  const size_t X,XY;
  Vector cell_length;
  
  static const int verbosity = 2;

  // converts global coordinate to global index
  size_t get_index( const Vector & position ) const
  {
    std::vector<size_t> x(Grid::dimensions);
    for(int d=0; d<Grid::dimensions; ++d)
	  {
		x[d] = (size_t)( position[d]/cell_length[d] );
	  }
	
    const size_t z_coord( Grid::dimensions == 3 ? x[2] : 0 );    
    const size_t index =  x[0] + x[1]*X + z_coord * XY ;
    return index;
  }

  
  // This keeps the particle on the domain boundary.
  bool constrain_position( Vector & position ) const 
  {
    bool flag = false;
    for( size_t i=0; i < position.size(); ++i )
	  {
		if(position[i] < 0)
		  {
			position[i] = 0;
			flag = true;
		  }
		else if( position[i] >= extent[i] )
		  {
			position[i] = extent[i]-1e-10;
			flag = true;
		  }
	  }
    return flag;
  }



public:

  // The constructor:
  ParticleTracker(
		  const Grid & _grid, 
		  const ConvectionField & _convection_field,
		  const DispersionField & _dispersion_field,
		  std::vector<Particle> & _particles
		  )
    : 
    grid(_grid), 
    convection_field(_convection_field),
    dispersion_field(_dispersion_field),
    particles(_particles),
    extent(grid.extent()),    
    X( grid.cells()[0] ),
    XY( grid.cells()[0]*grid.cells()[1] ),
    cell_length(Grid::dimensions)
  {
    for( unsigned int d=0; d < Grid::dimensions; ++d )
      {
	cell_length[ d ] = grid.extent()[ d ] / (double)grid.cells()[ d ];
      }
  }

  

  void get_concentration_field( Vector & concentration ) const
  {
    const double inv_particles_size = 1.0 / (double)particles.size();
	
    for( size_t p=0; p<particles.size(); ++p)
	  {
		const Vector & position = particles[p].position;

		// Convert global position to global index
		const size_t index = get_index( position );

		// Sum up all particles belonging to this position.
		// One particle gives one contribution to the concentration.
		concentration[ index ] += inv_particles_size;
	  }	  
  }


  

  size_t step( const double step_dx, const double target_time )
  {

    size_t still_moving = 0;


    for( size_t p = 0; p<particles.size(); ++p )
	  {

		// INSERT YOUR CODE HERE TO CALCULATE THE NEW LOCATIONS OF ALL PARTICLES:

		// random z-Vector with unit length
		const double pi = 3.14159265;
		const double phi = (double)rand()/(double)RAND_MAX * pi * 2.0;
		Vector random_jump(Grid::dimensions);
		random_jump[0]=cos(phi);
		random_jump[1]=sin(phi);

	  }// p-loop
		
    return still_moving;
  }
  
};

#endif // PARTICLETRACKER_HH
